//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include <JPEG.hpp>
#include <GIFImg.hpp>
#include <PngImage.hpp>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormMain *FormMain;
//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
String __fastcall BytesToStr(const TByteDynArray Value)
{
  String Result = "";
  for (int I = 0; I < Value.Length; ++I)
    Result += IntToHex(Value[I], 2) + " ";
  return Result;
}
//---------------------------------------------------------------------------
String __fastcall ToStr(TErrorType ErrorType)
{
  switch (ErrorType)
  {
    case etFormat:      return "Format error";
    case etChecksum:    return "Checksum error";
    case etUnsupported: return "Unsupported";
    default: /*etNone*/ return "No error";
  }
}
//---------------------------------------------------------------------------
String __fastcall ToStr(TContentType ContentType)
{
  switch (ContentType)
  {
    case ctBinary:      return "Binary";
    case ctMixed:       return "Mixed";
    case ctGS1:         return "GS1";
    case ctISO15434:    return "ISO 15434";
    case ctUnknownECI:  return "Unknown ECI";
    default: /*ctText*/ return "Text";
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
{
  BarcodeDecoder = new TBarcodeDecoder();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormDestroy(TObject *Sender)
{
  delete BarcodeDecoder;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::AddLine(const String Text)
{
  Memo->Lines->Add(Text);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonSelectPictureClick(TObject *Sender)
{
  if (OpenPictureDialog->Execute())
  {
    Image->Picture->LoadFromFile(OpenPictureDialog->FileName);
    DecodeBarcode();
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::DecodeBarcode()
{
  Memo->Lines->Clear();
  TopLeft.X = 0;
  TopLeft.Y = 0;
  TopRight.X = 0;
  TopRight.Y = 0;
  BottomLeft.X = 0;
  BottomLeft.Y = 0;
  BottomRight.X = 0;
  BottomRight.Y = 0;

  TDecodeResults Barcodes = BarcodeDecoder->Decode(Image->Picture->Graphic);
  if (Barcodes.Length == 0)
    AddLine("No barcode found");
  else
    for (int I = 0; I < Barcodes.Length; ++I)
    {
      Memo->Lines->BeginUpdate();
      try
      {
        TDecodeResult *Barcode = &Barcodes[I];
        AddLine("Barcode " + IntToStr(I + 1));
        if (Barcode->ErrorType != etNone)
          AddLine("  Error: " + ToStr(Barcode->ErrorType));
        if (Barcode->ErrorMessage.Length() > 0)
          AddLine("  Error message: " + Barcode->ErrorMessage);
        AddLine("  Format: " + Barcode->FormatName);
        AddLine("  Text: " + Barcode->Text);
        AddLine("  Rendered text: " + Barcode->RenderedText);
        AddLine("  Data: " + BytesToStr(Barcode->Data));
        AddLine("  Content type: " + ToStr(Barcode->ContentType));
        if (Barcode->Version.Length() > 0)
          AddLine("  Version: " + Barcode->Version);
        if (Barcode->EccLevel.Length() > 0)
          AddLine("  ECC level: " + Barcode->EccLevel);
        AddLine("  Has ECI: " + BoolToStr(Barcode->HasEci, true));
        AddLine("  ECI data: " + BytesToStr(Barcode->EciData));
        AddLine(String("  Position: ")
         + "(" + IntToStr((int)Barcode->TopLeft.X) + "," + IntToStr((int)Barcode->TopLeft.Y)
         + ") (" + IntToStr((int)Barcode->TopRight.X) + "," + IntToStr((int)Barcode->TopRight.Y)
         + ") (" + IntToStr((int)Barcode->BottomRight.X) + "," + IntToStr((int)Barcode->BottomRight.Y)
         + ") (" + IntToStr((int)Barcode->BottomLeft.X) + "," + IntToStr((int)Barcode->BottomLeft.Y) + ")");
        AddLine("  Lines: " + IntToStr(Barcode->LineCount));
        AddLine("  Orientation: " + IntToStr(Barcode->Orientation));
        AddLine("  Inverted: " + BoolToStr(Barcode->Inverted, True));
        AddLine("  Mirrored: " + BoolToStr(Barcode->Mirrored, True));
        AddLine("  Symbology identifier: " + Barcode->SymbologyIdentifier);
        if (Barcode->SequenceId.Length() > 0)
          AddLine("  Sequence ID: " + Barcode->SequenceId);
        if (Barcode->SequenceSize != -1)
          AddLine("  Sequence size: " + IntToStr(Barcode->SequenceSize));
        if (Barcode->SequenceIndex != -1)
          AddLine("  Sequence index: " + IntToStr(Barcode->SequenceIndex));
        AddLine();
        if (I == 0)
        {
          TopLeft = Barcode->TopLeft;
          TopRight = Barcode->TopRight;
          BottomLeft = Barcode->BottomLeft;
          BottomRight = Barcode->BottomRight;
        }
      }
      __finally
      {
        Memo->Lines->EndUpdate();
      }
    }

  PaintBox->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxAztecClick(TObject *Sender)
{
  if (CheckBoxAztec->Checked)
    BarcodeDecoder->Formats << foAztec;
  else
    BarcodeDecoder->Formats >> foAztec;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxCodabarClick(TObject *Sender)
{
  if (CheckBoxCodabar->Checked)
    BarcodeDecoder->Formats << foCodabar;
  else
    BarcodeDecoder->Formats >> foCodabar;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxCode39Click(TObject *Sender)
{
  if (CheckBoxCode39->Checked)
    BarcodeDecoder->Formats << foCode39;
  else
    BarcodeDecoder->Formats >> foCode39;
  DecodeBarcode();

}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxCode93Click(TObject *Sender)
{
  if (CheckBoxCode93->Checked)
    BarcodeDecoder->Formats << foCode93;
  else
    BarcodeDecoder->Formats >> foCode93;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxCode128Click(TObject *Sender)
{
  if (CheckBoxCode128->Checked)
    BarcodeDecoder->Formats << foCode128;
  else
    BarcodeDecoder->Formats >> foCode128;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxDataBarClick(TObject *Sender)
{
  if (CheckBoxDataBar->Checked)
    BarcodeDecoder->Formats << foDataBar;
  else
    BarcodeDecoder->Formats >> foDataBar;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxDataBarExpandedClick(TObject *Sender)
{
  if (CheckBoxDataBarExpanded->Checked)
    BarcodeDecoder->Formats << foDataBarExpanded;
  else
    BarcodeDecoder->Formats >> foDataBarExpanded;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxDataMatrixClick(TObject *Sender)
{
  if (CheckBoxDataMatrix->Checked)
    BarcodeDecoder->Formats << foDataMatrix;
  else
    BarcodeDecoder->Formats >> foDataMatrix;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxEan8Click(TObject *Sender)
{
  if (CheckBoxEan8->Checked)
    BarcodeDecoder->Formats << foEan8;
  else
    BarcodeDecoder->Formats >> foEan8;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxEan13Click(TObject *Sender)
{
  if (CheckBoxEan13->Checked)
    BarcodeDecoder->Formats << foEan13;
  else
    BarcodeDecoder->Formats >> foEan13;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxItfClick(TObject *Sender)
{
  if (CheckBoxItf->Checked)
    BarcodeDecoder->Formats << foItf;
  else
    BarcodeDecoder->Formats >> foItf;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxMaxiCodeClick(TObject *Sender)
{
  if (CheckBoxMaxiCode->Checked)
    BarcodeDecoder->Formats << foMaxiCode;
  else
    BarcodeDecoder->Formats >> foMaxiCode;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxPdf417Click(TObject *Sender)
{
  if (CheckBoxPdf417->Checked)
    BarcodeDecoder->Formats << foPdf417;
  else
    BarcodeDecoder->Formats >> foPdf417;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxQrCodeClick(TObject *Sender)
{
  if (CheckBoxQrCode->Checked)
    BarcodeDecoder->Formats << foQrCode;
  else
    BarcodeDecoder->Formats >> foQrCode;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxUpcAClick(TObject *Sender)
{
  if (CheckBoxUpcA->Checked)
    BarcodeDecoder->Formats << foUpcA;
  else
    BarcodeDecoder->Formats >> foUpcA;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxUpcEClick(TObject *Sender)
{
  if (CheckBoxUpcE->Checked)
    BarcodeDecoder->Formats << foUpcE;
  else
    BarcodeDecoder->Formats >> foUpcE;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxMicroQrCodeClick(TObject *Sender)
{
  if (CheckBoxMicroQrCode->Checked)
    BarcodeDecoder->Formats << foMicroQrCode;
  else
    BarcodeDecoder->Formats >> foMicroQrCode;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxDxFilmEdgeClick(TObject *Sender)
{
  if (CheckBoxDxFilmEdge->Checked)
    BarcodeDecoder->Formats << foDxFilmEdge;
  else
    BarcodeDecoder->Formats >> foDxFilmEdge;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxRMicroQrCodeClick(TObject *Sender)
{
  if (CheckBoxRMicroQrCode->Checked)
    BarcodeDecoder->Formats << foRMicroQrCode;
  else
    BarcodeDecoder->Formats >> foRMicroQrCode;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxShowPositionClick(TObject *Sender)
{
  PaintBox->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxTextModeChange(TObject *Sender)
{
  BarcodeDecoder->TextMode = (TTextMode)ComboBoxTextMode->ItemIndex;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxBinarizerChange(TObject *Sender)
{
  BarcodeDecoder->Binarizer = (TBinarizer)ComboBoxBinarizer->ItemIndex;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxEanAddOnChange(TObject *Sender)
{
  BarcodeDecoder->EanAddOn = (TEanAddOn)ComboBoxEanAddOn->ItemIndex;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::EditMinLineCountChange(TObject *Sender)
{
  BarcodeDecoder->MinLineCount = UpDownMinLineCount->Position;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::EditMaxBarcodesChange(TObject *Sender)
{
  BarcodeDecoder->MaxBarcodeCount = UpDownMaxBarcodes->Position;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxTryHarderClick(TObject *Sender)
{
  BarcodeDecoder->TryHarder = CheckBoxTryHarder->Checked;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxTryRotateClick(TObject *Sender)
{
  BarcodeDecoder->TryRotate = CheckBoxTryRotate->Checked;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxTryInvertClick(TObject *Sender)
{
  BarcodeDecoder->TryInvert = CheckBoxTryInvert->Checked;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxTryDownscaleClick(TObject *Sender)
{
  BarcodeDecoder->TryDownscale = CheckBoxTryDownscale->Checked;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxPureBarcodeClick(TObject *Sender)
{
  BarcodeDecoder->PureBarcode = CheckBoxPureBarcode->Checked;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxCodaBarStartEndClick(TObject *Sender)
{
  BarcodeDecoder->ReturnCodabarStartEnd = CheckBoxCodaBarStartEnd->Checked;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxValidateItfClick(TObject *Sender)
{
  BarcodeDecoder->ValidateITF = CheckBoxValidateItf->Checked;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxValidateCode39Click(TObject *Sender)
{
  BarcodeDecoder->ValidateCode39 = CheckBoxValidateCode39->Checked;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxCode39ExtendedClick(TObject *Sender)
{
  BarcodeDecoder->TryCode39ExtendedMode = CheckBoxCode39Extended->Checked;
  DecodeBarcode();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::PaintBoxPaint(TObject *Sender)
{
  if (CheckBoxShowPosition->Checked)
  {
    int X = (PaintBox->Width - Image->Picture->Width) / 2;
    int Y = (PaintBox->Height - Image->Picture->Height) / 2;
    PaintBox->Canvas->Pen->Color = clLime;
    PaintBox->Canvas->Pen->Width = 2;
    PaintBox->Canvas->MoveTo(X + TopLeft.X, Y + TopLeft.Y);
    PaintBox->Canvas->LineTo(X + TopRight.X, Y + TopRight.Y);
    PaintBox->Canvas->LineTo(X + BottomRight.X, Y + BottomRight.Y);
    PaintBox->Canvas->LineTo(X + BottomLeft.X, Y + BottomLeft.Y);
    PaintBox->Canvas->LineTo(X + TopLeft.X, Y + TopLeft.Y);
  }
}
//---------------------------------------------------------------------------

