//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)

#if _WIN64
#pragma link "mf.a"
#pragma link "mfplat.a"
#pragma link "mfplay.a"
#pragma link "mfreadwrite.a"
#pragma link "mfsrcsnk.a"
#else
#pragma link "mf.lib"
#pragma link "mfplat.lib"
#pragma link "mfplay.lib"
#pragma link "mfreadwrite.lib"
#pragma link "mfsrcsnk.lib"
#endif
#pragma link "MediaSuite"
#pragma link "BarcodeSuite"
#pragma resource "*.dfm"
TFormMain *FormMain;
//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
{
  Mediasuite::Startup();
  ClearTrackBars();
  BarcodeDecoder = new TBarcodeDecoder();
  BarcodeDecoder->MaxBarcodeCount = 1;
  BarcodeDecoder->Binarizer = biGlobalHistogram;
  Bitmap = new TBitmap();
  BitmapStream = new TMemoryStream();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormDestroy(TObject *Sender)
{
  delete BitmapStream;
  delete Bitmap;
  delete BarcodeDecoder;
  Mediasuite::Shutdown();
}
//---------------------------------------------------------------------------
String __fastcall SubtypeToStr(TGUID Subtype)
{
  if (IsEqualGUID(Subtype, MFVideoFormat_ARGB32))
    return "ARGB32";
  else if (IsEqualGUID(Subtype, MFVideoFormat_RGB32))
    return "RGB32";
  else if (IsEqualGUID(Subtype, MFVideoFormat_RGB24))
    return "RGB24";
  else if (IsEqualGUID(Subtype, MFVideoFormat_NV12))
    return "NV12";
  else if (IsEqualGUID(Subtype, MFVideoFormat_NV21))
    return "NV21";
  else if (IsEqualGUID(Subtype, MFVideoFormat_YUY2))
    return "YUY2";
  else if (IsEqualGUID(Subtype, MFVideoFormat_MJPG))
    return "MJPG";
  else
    return "";
}
//---------------------------------------------------------------------------
void __fastcall SetMediaTypes(TComboBox *ComboBox, TMediaTypes MediaTypes)
{
  ComboBox->Items->BeginUpdate();
  try
  {
    ComboBox->Items->Clear();
    for (int I = 0; I < MediaTypes.Length; ++I)
    {
      _di_IMediaType MediaType = MediaTypes[I];
      TSize FrameSize = MediaType->FrameSize;
      TRatio FrameRate = MediaType->FrameRate;
      if (FrameRate.Denominator == 0)
        FrameRate.Denominator = 1;
      ComboBox->Items->Add(SubtypeToStr(MediaType->Subtype) + " " + IntToStr((int)FrameSize.cx) + "x" + IntToStr((int)FrameSize.cy) + " " + IntToStr((int)(FrameRate.Numerator / FrameRate.Denominator)) + "Hz");
    }
  }
  __finally
  {
    ComboBox->Items->EndUpdate();
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::UpdateControls()
{
  ComboBoxFormat->Enabled = (CaptureEngine != NULL) && (CaptureEngine->State == saInitialized)
    && ((CaptureEngine->PreviewState == prDefault) || (CaptureEngine->PreviewState == prStarted));
  CheckBoxMirror->Enabled = (CaptureEngine != NULL) && (CaptureEngine->PreviewState == prStarted);
  ComboBoxRotation->Enabled = (CaptureEngine != NULL) && (CaptureEngine->PreviewState == prStarted);
  ButtonDefaultValues->Enabled = (CaptureEngine != NULL) && (CaptureEngine->PreviewState == prStarted);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CaptureEngineStateChanged(TObject *Sender)
{
  if (CaptureEngine->State == saInitialized)
  {
    PreviewMediaTypes = CaptureEngine->Source->VideoPreviewMediaTypes;
    SetMediaTypes(ComboBoxFormat, PreviewMediaTypes);
  }
  UpdateControls();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CaptureEnginePhotoStateChanged(TObject *Sender)
{
  TDecodeResults Barcodes;
  if (CaptureEngine->PhotoState == phTaken)
    if (CaptureEngine->PreviewState == prStarted)
    {
      Timer->Enabled = true;
      if (BitmapStream->Size > 0)
      try
      {
        BitmapStream->Position = 0;
        Bitmap->LoadFromStream(BitmapStream);
        Barcodes = BarcodeDecoder->Decode(Bitmap);
        if (Barcodes.Length > 0)
          LabelBarcode->Caption = Barcodes[0].FormatName + " " + Barcodes[0].Text;
      }
      catch (Exception &E)
      {
        LabelBarcode->Caption = "Error: " + E.Message;
      }
    }
  UpdateControls();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CaptureEnginePreviewStateChanged(TObject *Sender)
{
  if (CaptureEngine->PreviewState == prStarted)
  {
    CheckBoxMirrorClick(NULL); // set mirror
    ComboBoxRotationChange(NULL); // set rotation
    SetTrackBars();

    CaptureEngine->Source->PhotoMediaType = CaptureEngine->Source->VideoPreviewMediaType;
    Timer->Enabled = true;
  }
  else
    Timer->Enabled = false;
  UpdateControls();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxCameraDropDown(TObject *Sender)
{
  Devices = VideoCaptureSources();

  ComboBoxCamera->Items->BeginUpdate();
  try
  {
    String CurrentDeviceName = ComboBoxCamera->Text;
    int CurrentDeviceIndex = ComboBoxCamera->ItemIndex;

    ComboBoxCamera->Items->Clear();
    for (int i = 0; i < Devices.Length; ++i)
      ComboBoxCamera->Items->Add(Devices[i]->FriendlyName);
    ComboBoxCamera->Items->Add("");

    if ((CurrentDeviceIndex >= 0) && (CurrentDeviceIndex < ComboBoxCamera->Items->Count))
      if ((*ComboBoxCamera->Items)[CurrentDeviceIndex] == CurrentDeviceName)
      {
        ComboBoxCamera->ItemIndex = CurrentDeviceIndex;
        return;
      }

    ComboBoxCamera->ItemIndex = ComboBoxCamera->Items->IndexOf(CurrentDeviceName);
  }
  __finally
  {
    ComboBoxCamera->Items->EndUpdate();
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxCameraChange(TObject *Sender)
{
  Timer->Enabled = false;
  CaptureEngine = NULL;
  PreviewMediaTypes.Length = 0;
  ComboBoxFormat->Items->Clear();
  ClearTrackBars();
  if ((ComboBoxCamera->ItemIndex >= 0) && (ComboBoxCamera->ItemIndex < Devices.Length))
  {
    CaptureEngine = CreateCaptureEngine();
    CaptureEngine->OnPhotoStateChanged = CaptureEnginePhotoStateChanged;
    CaptureEngine->OnPreviewStateChanged = CaptureEnginePreviewStateChanged;
    CaptureEngine->OnStateChanged = CaptureEngineStateChanged;
    CaptureEngine->Initialize(true, Devices[ComboBoxCamera->ItemIndex], false);
  }
  UpdateControls();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxFormatChange(TObject *Sender)
{
  if (ComboBoxFormat->ItemIndex != -1)
  {
    if (CaptureEngine->PreviewState == prStarted)
      CaptureEngine->StopPreview();
    CaptureEngine->Source->VideoPreviewMediaType = PreviewMediaTypes[ComboBoxFormat->ItemIndex];
    CaptureEngine->StartPreview((THandle)PanelPreview->Handle);
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxMirrorClick(TObject *Sender)
{
  CaptureEngine->PreviewSink->Mirror = CheckBoxMirror->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxRotationChange(TObject *Sender)
{
  CaptureEngine->PreviewSink->Rotation[0] = 90 * ComboBoxRotation->ItemIndex;

  TSize FrameSize = PreviewMediaTypes[ComboBoxFormat->ItemIndex]->FrameSize;
  if ((ComboBoxRotation->ItemIndex == 0) || (ComboBoxRotation->ItemIndex == 2))
  {
    PanelPreview->Width = FrameSize.cx;
    PanelPreview->Height = FrameSize.cy;
  }
  else
  {
    PanelPreview->Width = FrameSize.cy;
    PanelPreview->Height = FrameSize.cx;
  }

  CaptureEngine->PreviewSink->UpdateVideo(PanelPreview->ClientRect);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::TimerTimer(TObject *Sender)
{
  if (CaptureEngine != NULL)
  {
    BitmapStream->Position = 0;
    CaptureEngine->TakePhoto(cfBmp, BitmapStream);
  }
  Timer->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall ClearTrackBar(TLabel *TrackbarLabel, TTrackBar *TrackBar, TCheckBox *CheckBoxAuto)
{
  TrackbarLabel->Enabled = false;
  TrackBar->Enabled = false;
  CheckBoxAuto->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ClearTrackBars()
{
  ClearTrackBar(LabelBacklightCompensation, TrackBarBacklightCompensation, CheckBoxBacklightCompensation);
  ClearTrackBar(LabelBrightness, TrackBarBrightness, CheckBoxBrightness);
  ClearTrackBar(LabelColorEnable, TrackBarColorEnable, CheckBoxColorEnable);
  ClearTrackBar(LabelContrast, TrackBarContrast, CheckBoxContrast);
  ClearTrackBar(LabelExposure, TrackBarExposure, CheckBoxExposure);
  ClearTrackBar(LabelFocus, TrackBarFocus, CheckBoxFocus);
  ClearTrackBar(LabelGain, TrackBarGain, CheckBoxGain);
  ClearTrackBar(LabelGamma, TrackBarGamma, CheckBoxGamma);
  ClearTrackBar(LabelHue, TrackBarHue, CheckBoxHue);
  ClearTrackBar(LabelIris, TrackBarIris, CheckBoxIris);
  ClearTrackBar(LabelPan, TrackBarPan, CheckBoxPan);
  ClearTrackBar(LabelRoll, TrackBarRoll, CheckBoxRoll);
  ClearTrackBar(LabelSaturation, TrackBarSaturation, CheckBoxSaturation);
  ClearTrackBar(LabelSharpness, TrackBarSharpness, CheckBoxSharpness);
  ClearTrackBar(LabelTilt, TrackBarTilt, CheckBoxTilt);
  ClearTrackBar(LabelWhiteBalance, TrackBarWhiteBalance, CheckBoxWhiteBalance);
  ClearTrackBar(LabelZoom, TrackBarZoom, CheckBoxZoom);
}
//---------------------------------------------------------------------------
void __fastcall SetTrackBar(TLabel *TrackbarLabel, TTrackBar *TrackBar, TCheckBox *CheckBoxAuto, int Position, TVideoProcAmpRange Range, bool Auto)
{
  CheckBoxAuto->Checked = Auto;
  TrackBar->Min = Range.Min;
  TrackBar->Max = Range.Max;
  TrackBar->PageSize = Range.Delta;
  if ((Range.Delta > 0) && ((Range.Max - Range.Min) / Range.Delta > 25))
    TrackBar->Frequency = (Range.Max - Range.Min) / 25;
  else
    TrackBar->Frequency = 1;
  TrackBar->Position = Position;

  TrackbarLabel->Enabled = true;
  CheckBoxAuto->Enabled = Range.Auto && Range.Manual;
  TrackBar->Enabled = !CheckBoxAuto->Checked;
}
//---------------------------------------------------------------------------
void __fastcall SetTrackBar(TLabel *TrackbarLabel, TTrackBar *TrackBar, TCheckBox *CheckBoxAuto, int Position, TCameraControlRange Range, bool Auto)
{
  CheckBoxAuto->Checked = Auto;
  TrackBar->Min = Range.Min;
  TrackBar->Max = Range.Max;
  TrackBar->PageSize = Range.Delta;
  if ((Range.Delta > 0) && ((Range.Max - Range.Min) / Range.Delta > 25))
    TrackBar->Frequency = (Range.Max - Range.Min) / 25;
  else
    TrackBar->Frequency = 1;
  TrackBar->Position = Position;

  TrackbarLabel->Enabled = true;
  CheckBoxAuto->Enabled = Range.Auto && Range.Manual;
  TrackBar->Enabled = !CheckBoxAuto->Checked;
}
//---------------------------------------------------------------------------
bool TrackBarsSetting;

void __fastcall TFormMain::SetTrackBars()
{
  if (!TrackBarsSetting)
    try
    {
      TrackBarsSetting = true;
      ClearTrackBars();

      _di_IVideoProcAmp VideoProcAmp = CaptureEngine->Source->VideoProcAmp;
      if (VideoProcAmp != NULL)
      {
        if (VideoProcAmp->BacklightCompensationSupported)
          SetTrackBar(LabelBacklightCompensation, TrackBarBacklightCompensation, CheckBoxBacklightCompensation, VideoProcAmp->BacklightCompensation, VideoProcAmp->BacklightCompensationRange, VideoProcAmp->BacklightCompensationAuto);

        if (VideoProcAmp->BrightnessSupported)
          SetTrackBar(LabelBrightness, TrackBarBrightness, CheckBoxBrightness, VideoProcAmp->Brightness, VideoProcAmp->BrightnessRange, VideoProcAmp->BrightnessAuto);

        if (VideoProcAmp->ColorEnableSupported)
          SetTrackBar(LabelColorEnable, TrackBarColorEnable, CheckBoxColorEnable, VideoProcAmp->ColorEnable, VideoProcAmp->ColorEnableRange, VideoProcAmp->ColorEnableAuto);

        if (VideoProcAmp->ContrastSupported)
          SetTrackBar(LabelContrast, TrackBarContrast, CheckBoxContrast, VideoProcAmp->Contrast, VideoProcAmp->ContrastRange, VideoProcAmp->ContrastAuto);

        if (VideoProcAmp->GainSupported)
          SetTrackBar(LabelGain, TrackBarGain, CheckBoxGain, VideoProcAmp->Gain, VideoProcAmp->GainRange, VideoProcAmp->GainAuto);

        if (VideoProcAmp->GammaSupported)
          SetTrackBar(LabelGamma, TrackBarGamma, CheckBoxGamma, VideoProcAmp->Gamma, VideoProcAmp->GammaRange, VideoProcAmp->GammaAuto);

        if (VideoProcAmp->HueSupported)
          SetTrackBar(LabelHue, TrackBarHue, CheckBoxHue, VideoProcAmp->Hue, VideoProcAmp->HueRange, VideoProcAmp->HueAuto);

        if (VideoProcAmp->SaturationSupported)
          SetTrackBar(LabelSaturation, TrackBarSaturation, CheckBoxSaturation, VideoProcAmp->Saturation, VideoProcAmp->SaturationRange, VideoProcAmp->SaturationAuto);

        if (VideoProcAmp->SharpnessSupported)
          SetTrackBar(LabelSharpness, TrackBarSharpness, CheckBoxSharpness, VideoProcAmp->Sharpness, VideoProcAmp->SharpnessRange, VideoProcAmp->SharpnessAuto);

        if (VideoProcAmp->WhiteBalanceSupported)
          SetTrackBar(LabelWhiteBalance, TrackBarWhiteBalance, CheckBoxWhiteBalance, VideoProcAmp->WhiteBalance, VideoProcAmp->WhiteBalanceRange, VideoProcAmp->WhiteBalanceAuto);
      }

      _di_ICameraControl CameraControl = CaptureEngine->Source->CameraControl;
      if (CameraControl != NULL)
      {
        if (CameraControl->ExposureSupported)
          SetTrackBar(LabelExposure, TrackBarExposure, CheckBoxExposure, CameraControl->Exposure, CameraControl->ExposureRange, CameraControl->ExposureAuto);

        if (CameraControl->FocusSupported)
          SetTrackBar(LabelFocus, TrackBarFocus, CheckBoxFocus, CameraControl->Focus, CameraControl->FocusRange, CameraControl->FocusAuto);

        if (CameraControl->IrisSupported)
          SetTrackBar(LabelIris, TrackBarIris, CheckBoxIris, CameraControl->Iris, CameraControl->IrisRange, CameraControl->IrisAuto);

        if (CameraControl->PanSupported)
          SetTrackBar(LabelPan, TrackBarPan, CheckBoxPan, CameraControl->Pan, CameraControl->PanRange, CameraControl->PanAuto);

        if (CameraControl->RollSupported)
          SetTrackBar(LabelRoll, TrackBarRoll, CheckBoxRoll, CameraControl->Roll, CameraControl->RollRange, CameraControl->RollAuto);

        if (CameraControl->TiltSupported)
          SetTrackBar(LabelTilt, TrackBarTilt, CheckBoxTilt, CameraControl->Tilt, CameraControl->TiltRange, CameraControl->TiltAuto);

        if (CameraControl->ZoomSupported)
          SetTrackBar(LabelZoom, TrackBarZoom, CheckBoxZoom, CameraControl->Zoom, CameraControl->ZoomRange, CameraControl->ZoomAuto);
      }
    }
    __finally
    {
      TrackBarsSetting = false;
    }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxClick(TObject *Sender)
{
  if (!TrackBarsSetting)
    try
    {
      _di_IVideoProcAmp VideoProcAmp = CaptureEngine->Source->VideoProcAmp;
      if (VideoProcAmp != NULL)
      {
        if (Sender == CheckBoxBacklightCompensation)
          VideoProcAmp->BacklightCompensationAuto = CheckBoxBacklightCompensation->Checked;
        else if (Sender == CheckBoxBrightness)
          VideoProcAmp->BrightnessAuto = CheckBoxBrightness->Checked;
        else if (Sender == CheckBoxColorEnable)
          VideoProcAmp->ColorEnableAuto = CheckBoxColorEnable->Checked;
        else if (Sender == CheckBoxContrast)
          VideoProcAmp->ContrastAuto = CheckBoxContrast->Checked;
        else if (Sender == CheckBoxGain)
          VideoProcAmp->GainAuto = CheckBoxGain->Checked;
        else if (Sender == CheckBoxGamma)
          VideoProcAmp->GammaAuto = CheckBoxGamma->Checked;
        else if (Sender == CheckBoxHue)
          VideoProcAmp->HueAuto = CheckBoxHue->Checked;
        else if (Sender == CheckBoxSaturation)
          VideoProcAmp->SaturationAuto = CheckBoxSaturation->Checked;
        else if (Sender == CheckBoxSharpness)
          VideoProcAmp->SharpnessAuto = CheckBoxSharpness->Checked;
        else if (Sender == CheckBoxWhiteBalance)
          VideoProcAmp->WhiteBalanceAuto = CheckBoxWhiteBalance->Checked;
      }

      _di_ICameraControl CameraControl = CaptureEngine->Source->CameraControl;
      if (CameraControl != NULL)
      {
        if (Sender == CheckBoxExposure)
          CameraControl->ExposureAuto = CheckBoxExposure->Checked;
        else if (Sender == CheckBoxFocus)
          CameraControl->FocusAuto = CheckBoxFocus->Checked;
        else if (Sender == CheckBoxIris)
          CameraControl->IrisAuto = CheckBoxIris->Checked;
        else if (Sender == CheckBoxPan)
          CameraControl->PanAuto = CheckBoxPan->Checked;
        else if (Sender == CheckBoxRoll)
          CameraControl->RollAuto = CheckBoxRoll->Checked;
        else if (Sender == CheckBoxTilt)
          CameraControl->TiltAuto = CheckBoxTilt->Checked;
        else if (Sender == CheckBoxZoom)
          CameraControl->ZoomAuto = CheckBoxZoom->Checked;
      }
    }
    __finally
    {
      SetTrackBars();
    }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::TrackBarChange(TObject *Sender)
{
  if (!TrackBarsSetting)
  {
    _di_IVideoProcAmp VideoProcAmp = CaptureEngine->Source->VideoProcAmp;
    if (VideoProcAmp != NULL)
    {
      if (Sender == TrackBarBacklightCompensation)
        VideoProcAmp->BacklightCompensation = TrackBarBacklightCompensation->Position;
      else if (Sender == TrackBarBrightness)
        VideoProcAmp->Brightness = TrackBarBrightness->Position;
      else if (Sender == TrackBarColorEnable)
        VideoProcAmp->ColorEnable = TrackBarColorEnable->Position;
      else if (Sender == TrackBarContrast)
        VideoProcAmp->Contrast = TrackBarContrast->Position;
      else if (Sender == TrackBarGain)
        VideoProcAmp->Gain = TrackBarGain->Position;
      else if (Sender == TrackBarGamma)
        VideoProcAmp->Gamma = TrackBarGamma->Position;
      else if (Sender == TrackBarHue)
        VideoProcAmp->Hue = TrackBarHue->Position;
      else if (Sender == TrackBarSaturation)
        VideoProcAmp->Saturation = TrackBarSaturation->Position;
      else if (Sender == TrackBarSharpness)
        VideoProcAmp->Sharpness = TrackBarSharpness->Position;
      else if (Sender == TrackBarWhiteBalance)
        VideoProcAmp->WhiteBalance = TrackBarWhiteBalance->Position;
    }

    _di_ICameraControl CameraControl = CaptureEngine->Source->CameraControl;
    if (CameraControl != NULL)
    {
      if (Sender == TrackBarExposure)
        CameraControl->Exposure = TrackBarExposure->Position;
      else if (Sender == TrackBarFocus)
        CameraControl->Focus = TrackBarFocus->Position;
      else if (Sender == TrackBarIris)
        CameraControl->Iris = TrackBarIris->Position;
      else if (Sender == TrackBarPan)
        CameraControl->Pan = TrackBarPan->Position;
      else if (Sender == TrackBarRoll)
        CameraControl->Roll = TrackBarRoll->Position;
      else if (Sender == TrackBarTilt)
        CameraControl->Tilt = TrackBarTilt->Position;
      else if (Sender == TrackBarZoom)
        CameraControl->Zoom = TrackBarZoom->Position;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonDefaultValuesClick(TObject *Sender)
{
  try
  {
    _di_IVideoProcAmp VideoProcAmp = CaptureEngine->Source->VideoProcAmp;
    if (VideoProcAmp != NULL)
    {
      if (VideoProcAmp->BacklightCompensationSupported)
        if (!VideoProcAmp->BacklightCompensationAuto)
          VideoProcAmp->BacklightCompensation = VideoProcAmp->BacklightCompensationRange.Default;

      if (VideoProcAmp->BrightnessSupported)
        if (!VideoProcAmp->BrightnessAuto)
          VideoProcAmp->Brightness = VideoProcAmp->BrightnessRange.Default;

      if (VideoProcAmp->ColorEnableSupported)
        if (!VideoProcAmp->ColorEnableAuto)
          VideoProcAmp->ColorEnable = VideoProcAmp->ColorEnableRange.Default;

      if (VideoProcAmp->ContrastSupported)
        if (!VideoProcAmp->ContrastAuto)
          VideoProcAmp->Contrast = VideoProcAmp->ContrastRange.Default;

      if (VideoProcAmp->GainSupported)
        if (!VideoProcAmp->GainAuto)
          VideoProcAmp->Gain = VideoProcAmp->GainRange.Default;

      if (VideoProcAmp->GammaSupported)
        if (!VideoProcAmp->GammaAuto)
          VideoProcAmp->Gamma = VideoProcAmp->GammaRange.Default;

      if (VideoProcAmp->HueSupported)
        if (!VideoProcAmp->HueAuto)
          VideoProcAmp->Hue = VideoProcAmp->HueRange.Default;

      if (VideoProcAmp->SaturationSupported)
        if (!VideoProcAmp->SaturationAuto)
          VideoProcAmp->Saturation = VideoProcAmp->SaturationRange.Default;

      if (VideoProcAmp->SharpnessSupported)
        if (!VideoProcAmp->SharpnessAuto)
          VideoProcAmp->Sharpness = VideoProcAmp->SharpnessRange.Default;

      if (VideoProcAmp->WhiteBalanceSupported)
        if (!VideoProcAmp->WhiteBalanceAuto)
          VideoProcAmp->WhiteBalance = VideoProcAmp->WhiteBalanceRange.Default;
    }

    _di_ICameraControl CameraControl = CaptureEngine->Source->CameraControl;
    if (CameraControl != NULL)
    {
      if (CameraControl->ExposureSupported)
        if (!CameraControl->ExposureAuto)
          CameraControl->Exposure = CameraControl->ExposureRange.Default;

      if (CameraControl->FocusSupported)
        if (!CameraControl->FocusAuto)
          CameraControl->Focus = CameraControl->FocusRange.Default;

      if (CameraControl->IrisSupported)
        if (!CameraControl->IrisAuto)
          CameraControl->Iris = CameraControl->IrisRange.Default;

      if (CameraControl->PanSupported)
        if (!CameraControl->PanAuto)
          CameraControl->Pan = CameraControl->PanRange.Default;

      if (CameraControl->RollSupported)
        if (!CameraControl->RollAuto)
          CameraControl->Roll = CameraControl->RollRange.Default;

      if (CameraControl->TiltSupported)
        if (!CameraControl->TiltAuto)
          CameraControl->Tilt = CameraControl->TiltRange.Default;

      if (CameraControl->ZoomSupported)
        if (!CameraControl->ZoomAuto)
          CameraControl->Zoom = CameraControl->ZoomRange.Default;
    }
  }
  __finally
  {
    SetTrackBars();
  }
}
//---------------------------------------------------------------------------

