unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, XPMan, ComCtrls, MediaFoundation, MediaSuite,
  BarcodeSuite;

type
  TFormMain = class(TForm)
    PanelTop: TPanel;
    PanelRight: TPanel;
    PanelCenter: TPanel;
    LabelCamera: TLabel;
    ComboBoxCamera: TComboBox;
    LabelFormat: TLabel;
    ComboBoxFormat: TComboBox;
    TrackBarBacklightCompensation: TTrackBar;
    LabelBacklightCompensation: TLabel;
    CheckBoxBacklightCompensation: TCheckBox;
    LabelBrightness: TLabel;
    TrackBarBrightness: TTrackBar;
    CheckBoxBrightness: TCheckBox;
    LabelColorEnable: TLabel;
    TrackBarColorEnable: TTrackBar;
    CheckBoxColorEnable: TCheckBox;
    LabelContrast: TLabel;
    TrackBarContrast: TTrackBar;
    CheckBoxContrast: TCheckBox;
    LabelExposure: TLabel;
    TrackBarExposure: TTrackBar;
    CheckBoxExposure: TCheckBox;
    LabelFocus: TLabel;
    TrackBarFocus: TTrackBar;
    CheckBoxFocus: TCheckBox;
    LabelGain: TLabel;
    TrackBarGain: TTrackBar;
    CheckBoxGain: TCheckBox;
    LabelGamma: TLabel;
    TrackBarGamma: TTrackBar;
    CheckBoxGamma: TCheckBox;
    LabelHue: TLabel;
    TrackBarHue: TTrackBar;
    CheckBoxHue: TCheckBox;
    LabelIris: TLabel;
    TrackBarIris: TTrackBar;
    CheckBoxIris: TCheckBox;
    LabelPan: TLabel;
    TrackBarPan: TTrackBar;
    CheckBoxPan: TCheckBox;
    LabelRoll: TLabel;
    TrackBarRoll: TTrackBar;
    CheckBoxRoll: TCheckBox;
    LabelSaturation: TLabel;
    TrackBarSaturation: TTrackBar;
    CheckBoxSaturation: TCheckBox;
    LabelSharpness: TLabel;
    TrackBarSharpness: TTrackBar;
    CheckBoxSharpness: TCheckBox;
    LabelTilt: TLabel;
    TrackBarTilt: TTrackBar;
    CheckBoxTilt: TCheckBox;
    LabelWhiteBalance: TLabel;
    TrackBarWhiteBalance: TTrackBar;
    CheckBoxWhiteBalance: TCheckBox;
    LabelZoom: TLabel;
    TrackBarZoom: TTrackBar;
    CheckBoxZoom: TCheckBox;
    CheckBoxAspectRatio: TCheckBox;
    ButtonDefaultValues: TButton;
    ColorBoxBorderColor: TColorBox;
    LabelBorderColor: TLabel;
    PaintBox: TPaintBox;
    CheckBoxStretch: TCheckBox;
    ButtonSaveCurrentImage: TButton;
    LabelBarcode: TLabel;
    Timer: TTimer;
    procedure ComboBoxCameraDropDown(Sender: TObject);
    procedure ComboBoxCameraChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ComboBoxFormatChange(Sender: TObject);
    procedure CheckBoxClick(Sender: TObject);
    procedure TrackBarChange(Sender: TObject);
    procedure ButtonSaveCurrentImageClick(Sender: TObject);
    procedure ButtonDefaultValuesClick(Sender: TObject);
    procedure ColorBoxBorderColorChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBoxPaint(Sender: TObject);
    procedure CheckBoxStretchClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TimerTimer(Sender: TObject);
  private
    { Private declarations }
    Bitmap: TBitmap;
    Devices: TIActivateDynArray;
    MediaTypes: TMediaTypes;
    VideoCapture: TVideoCapture;
    BarcodeDecoder: TBarcodeDecoder;
    procedure ClearBitmap;
    procedure ClearTrackBars;
    procedure SetTrackBars;
    procedure OnSample(Sender: TObject; TimeStamp: Int64; const Sample: ISample);
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}

procedure TFormMain.FormCreate(Sender: TObject);
begin
  MediaSuite.Startup;
  Bitmap := TBitmap.Create;
  BarcodeDecoder := TBarcodeDecoder.Create;
  BarcodeDecoder.MaxBarcodeCount := 1;
  BarcodeDecoder.Binarizer := biGlobalHistogram;
  VideoCapture := TVideoCapture.Create;
  VideoCapture.OnSample := OnSample;
  PanelCenter.DoubleBuffered := True;
  ClearTrackBars;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  VideoCapture.Free;
  BarcodeDecoder.Free;
  Bitmap.Free;
  MediaSuite.Shutdown;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  VideoCapture.Active := False;
end;

procedure TFormMain.OnSample(Sender: TObject; TimeStamp: Int64; const Sample: ISample);
begin
  if VideoCapture.ToBitmap(Sample, Bitmap) then
    PaintBox.Invalidate // repaint image
  else
    ClearBitmap;

  if ComboBoxFormat.ItemIndex <> -1 then
    if VideoCapture.Active then
      VideoCapture.ReadSample; // read next sample
end;

procedure TFormMain.TimerTimer(Sender: TObject);
var Barcodes: TDecodeResults;
begin
  Barcodes := nil; // to avoid warning
  if VideoCapture.Active then
  try
    if (Bitmap.Width > 0) and (Bitmap.Height > 0) then
    begin
      Barcodes := BarcodeDecoder.Decode(Bitmap);
      if Barcodes <> nil then
        with Barcodes[0] do
          if ErrorType = etNone then
            LabelBarcode.Caption := FormatName + ': ' + Text;
    end
  except
    on E: Exception do
      LabelBarcode.Caption := 'Error: ' + E.Message;
  end;
end;

procedure TFormMain.ComboBoxCameraDropDown(Sender: TObject);
var
  CurrentDeviceName: string;
  CurrentDeviceIndex: Integer;
  I: Integer;
begin
  ComboBoxCamera.Items.BeginUpdate;
  try
    CurrentDeviceName := ComboBoxCamera.Text;
    CurrentDeviceIndex := ComboBoxCamera.ItemIndex;
    Devices := VideoCapture.Devices;

    ComboBoxCamera.Items.Clear;
    for I := 0 to Length(Devices) - 1 do
      ComboBoxCamera.Items.Add(Devices[I].FriendlyName);
    ComboBoxCamera.Items.Add('');

    if (CurrentDeviceIndex >= 0) and (CurrentDeviceIndex < ComboBoxCamera.Items.Count) then
      if ComboBoxCamera.Items[CurrentDeviceIndex] = CurrentDeviceName then
      begin
        ComboBoxCamera.ItemIndex := CurrentDeviceIndex;
        Exit;
      end;

    ComboBoxCamera.ItemIndex := ComboBoxCamera.Items.IndexOf(CurrentDeviceName);
  finally
    ComboBoxCamera.Items.EndUpdate;
  end;
end;

function SubtypeToStr(Subtype: TGUID): string;
begin
  if IsEqualGUID(Subtype, MFVideoFormat_ARGB32) then
    Result := 'ARGB32'
  else if IsEqualGUID(Subtype, MFVideoFormat_RGB32) then
    Result := 'RGB32'
  else if IsEqualGUID(Subtype, MFVideoFormat_RGB24) then
    Result := 'RGB24'
  else if IsEqualGUID(Subtype, MFVideoFormat_NV12) then
    Result := 'NV12'
  else if IsEqualGUID(Subtype, MFVideoFormat_NV21) then
    Result := 'NV21'
  else if IsEqualGUID(Subtype, MFVideoFormat_YUY2) then
    Result := 'YUY2'
  else if IsEqualGUID(Subtype, MFVideoFormat_MJPG) then
    Result := 'MJPG'
  else
    Result := '';
end;

procedure TFormMain.ClearBitmap;
begin
  Bitmap.Width := 0;
  Bitmap.Height := 0;
  PaintBox.Invalidate;
end;

procedure TFormMain.ComboBoxCameraChange(Sender: TObject);
var
  I: Integer;
  MediaType: IMediaType;
  FrameSize: TSize;
  FrameRate: TRatio;
begin
  Timer.Enabled := False;
  MediaTypes := nil;
  ComboBoxFormat.Enabled := False;
  ButtonDefaultValues.Enabled := False;
  ButtonSaveCurrentImage.Enabled := False;
  ClearTrackBars;
  VideoCapture.Active := False;
  ComboBoxFormat.ItemIndex := -1;
  ClearBitmap;
  if (ComboBoxCamera.ItemIndex < 0) or (ComboBoxCamera.ItemIndex >= Length(Devices)) then
    Exit;

  VideoCapture.Device := Devices[ComboBoxCamera.ItemIndex];
  VideoCapture.Active := True;

  ComboBoxFormat.Items.BeginUpdate;
  try
    ComboBoxFormat.Enabled := True;
    ComboBoxFormat.Items.Clear;
    MediaTypes := VideoCapture.MediaTypes;
    for I := 0 to Length(MediaTypes) - 1 do
    begin
      MediaType := MediaTypes[I];
      FrameSize := MediaType.FrameSize;
      FrameRate := MediaType.FrameRate;
      if FrameRate.Denominator = 0 then
        FrameRate.Denominator := 1;
      ComboBoxFormat.Items.Add(SubtypeToStr(MediaType.Subtype) + ' ' + IntToStr(FrameSize.cx) + 'x' + IntToStr(FrameSize.cy) + ' ' + IntToStr(FrameRate.Numerator div FrameRate.Denominator) + 'Hz');
    end;
  finally
    ComboBoxFormat.Items.EndUpdate;
  end;
end;

procedure TFormMain.ComboBoxFormatChange(Sender: TObject);
begin
  if ComboBoxFormat.ItemIndex <> -1 then
  begin
    VideoCapture.Active := False;
    VideoCapture.Active := True;
    VideoCapture.MediaType := MediaTypes[ComboBoxFormat.ItemIndex];
    VideoCapture.ReadSample;
    ButtonDefaultValues.Enabled := True;
    ButtonSaveCurrentImage.Enabled := True;
    SetTrackBars;
    Timer.Enabled := True;
  end;
end;

procedure TFormMain.CheckBoxStretchClick(Sender: TObject);
begin
  CheckBoxAspectRatio.Enabled := CheckBoxStretch.Checked;
end;

procedure TFormMain.ColorBoxBorderColorChange(Sender: TObject);
begin
  PanelCenter.Color := ColorBoxBorderColor.Selected;
end;

procedure TFormMain.ButtonSaveCurrentImageClick(Sender: TObject);
begin
  Bitmap.SaveToFile('image_' + FormatDateTime('yyyymmdd_hhnnss', Now) + '.bmp');
end;

procedure TFormMain.PaintBoxPaint(Sender: TObject);
var
  AspectRatio: Double;
  Height, Width: Integer;
  Rect: TRect;
begin
  if Bitmap.Empty then
    PaintBox.Canvas.FillRect(PaintBox.ClientRect)
  else if CheckBoxStretch.Checked then
  begin
    if CheckBoxAspectRatio.Checked then
    begin
      if Bitmap.Width / Bitmap.Height >= PaintBox.Width / PaintBox.Height then
      begin
        AspectRatio := PaintBox.Width / Bitmap.Width;
        Height := Round(AspectRatio * Bitmap.Height);
        Rect.Left := 0;
        Rect.Top := (PaintBox.Height - Height) div 2;
        Rect.Right := PaintBox.Width;
        Rect.Bottom := Rect.Top + Height;
      end
      else
      begin
        AspectRatio := PaintBox.Height / Bitmap.Height;
        Width := Round(AspectRatio * Bitmap.Width);
        Rect.Left := (PaintBox.Width - Width) div 2;
        Rect.Top := 0;
        Rect.Right := Rect.Left + Width;
        Rect.Bottom := PaintBox.Height;
      end;

      PaintBox.Canvas.StretchDraw(Rect,  Bitmap);
    end
    else
      Rect := PaintBox.ClientRect;

    PaintBox.Canvas.StretchDraw(Rect, Bitmap);
  end
  else
  begin
    PaintBox.Canvas.Draw(
      (PaintBox.Width - Bitmap.Width) div 2,
      (PaintBox.Height - Bitmap.Height) div 2,
      Bitmap);
  end;
end;

procedure ClearTrackBar(TrackbarLabel: TLabel; TrackBar: TTrackBar; CheckBoxAuto: TCheckBox);
begin
  TrackbarLabel.Enabled := False;
  TrackBar.Enabled := False;
  CheckBoxAuto.Enabled := False;
end;

procedure TFormMain.ClearTrackBars;
begin
  ClearTrackBar(LabelBacklightCompensation, TrackBarBacklightCompensation, CheckBoxBacklightCompensation);
  ClearTrackBar(LabelBrightness, TrackBarBrightness, CheckBoxBrightness);
  ClearTrackBar(LabelColorEnable, TrackBarColorEnable, CheckBoxColorEnable);
  ClearTrackBar(LabelContrast, TrackBarContrast, CheckBoxContrast);
  ClearTrackBar(LabelExposure, TrackBarExposure, CheckBoxExposure);
  ClearTrackBar(LabelFocus, TrackBarFocus, CheckBoxFocus);
  ClearTrackBar(LabelGain, TrackBarGain, CheckBoxGain);
  ClearTrackBar(LabelGamma, TrackBarGamma, CheckBoxGamma);
  ClearTrackBar(LabelHue, TrackBarHue, CheckBoxHue);
  ClearTrackBar(LabelIris, TrackBarIris, CheckBoxIris);
  ClearTrackBar(LabelPan, TrackBarPan, CheckBoxPan);
  ClearTrackBar(LabelRoll, TrackBarRoll, CheckBoxRoll);
  ClearTrackBar(LabelSaturation, TrackBarSaturation, CheckBoxSaturation);
  ClearTrackBar(LabelSharpness, TrackBarSharpness, CheckBoxSharpness);
  ClearTrackBar(LabelTilt, TrackBarTilt, CheckBoxTilt);
  ClearTrackBar(LabelWhiteBalance, TrackBarWhiteBalance, CheckBoxWhiteBalance);
  ClearTrackBar(LabelZoom, TrackBarZoom, CheckBoxZoom);
end;

procedure SetTrackBar(TrackbarLabel: TLabel; TrackBar: TTrackBar; CheckBoxAuto: TCheckBox; Position: Integer; Range: TVideoProcAmpRange; Auto: Boolean); overload;
begin
  CheckBoxAuto.Checked := Auto;
  TrackBar.Min := Range.Min;
  TrackBar.Max := Range.Max;
  TrackBar.PageSize := Range.Delta;
  if (Range.Delta > 0) and ((Range.Max - Range.Min) div Range.Delta > 25) then
    TrackBar.Frequency := (Range.Max - Range.Min) div 25
  else
    TrackBar.Frequency := 1;
  TrackBar.Position := Position;
  TrackbarLabel.Enabled := True;
  CheckBoxAuto.Enabled := Range.Auto and Range.Manual;
  TrackBar.Enabled := not CheckBoxAuto.Checked;
end;

procedure SetTrackBar(TrackbarLabel: TLabel; TrackBar: TTrackBar; CheckBoxAuto: TCheckBox; Position: Integer; Range: TCameraControlRange; Auto: Boolean); overload;
begin
  CheckBoxAuto.Checked := Auto;
  TrackBar.Min := Range.Min;
  TrackBar.Max := Range.Max;
  TrackBar.PageSize := Range.Delta;
  if (Range.Delta > 0) and ((Range.Max - Range.Min) div Range.Delta > 25) then
    TrackBar.Frequency := (Range.Max - Range.Min) div 25
  else
    TrackBar.Frequency := 1;
  TrackBar.Position := Position;
  TrackbarLabel.Enabled := True;
  CheckBoxAuto.Enabled := Range.Auto and Range.Manual;
  TrackBar.Enabled := not CheckBoxAuto.Checked;
end;

var
  TrackBarsSetting: Boolean;

procedure TFormMain.SetTrackBars;
var
  VideoProcAmp: IVideoProcAmp;
  CameraControl: ICameraControl;
begin
  if not TrackBarsSetting then
  try
    TrackBarsSetting := True;
    ClearTrackBars;

    VideoProcAmp := VideoCapture.VideoProcAmp;
    if VideoProcAmp <> nil then
    begin
      if VideoProcAmp.BacklightCompensationSupported then
        SetTrackBar(LabelBacklightCompensation, TrackBarBacklightCompensation, CheckBoxBacklightCompensation, VideoProcAmp.BacklightCompensation, VideoProcAmp.BacklightCompensationRange, VideoProcAmp.BacklightCompensationAuto);

      if VideoProcAmp.BrightnessSupported then
        SetTrackBar(LabelBrightness, TrackBarBrightness, CheckBoxBrightness, VideoProcAmp.Brightness, VideoProcAmp.BrightnessRange, VideoProcAmp.BrightnessAuto);

      if VideoProcAmp.ColorEnableSupported then
        SetTrackBar(LabelColorEnable, TrackBarColorEnable, CheckBoxColorEnable, VideoProcAmp.ColorEnable, VideoProcAmp.ColorEnableRange, VideoProcAmp.ColorEnableAuto);

      if VideoProcAmp.ContrastSupported then
        SetTrackBar(LabelContrast, TrackBarContrast, CheckBoxContrast, VideoProcAmp.Contrast, VideoProcAmp.ContrastRange, VideoProcAmp.ContrastAuto);

      if VideoProcAmp.GainSupported then
        SetTrackBar(LabelGain, TrackBarGain, CheckBoxGain, VideoProcAmp.Gain, VideoProcAmp.GainRange, VideoProcAmp.GainAuto);

      if VideoProcAmp.GammaSupported then
        SetTrackBar(LabelGamma, TrackBarGamma, CheckBoxGamma, VideoProcAmp.Gamma, VideoProcAmp.GammaRange, VideoProcAmp.GammaAuto);

      if VideoProcAmp.HueSupported then
        SetTrackBar(LabelHue, TrackBarHue, CheckBoxHue, VideoProcAmp.Hue, VideoProcAmp.HueRange, VideoProcAmp.HueAuto);

      if VideoProcAmp.SaturationSupported then
        SetTrackBar(LabelSaturation, TrackBarSaturation, CheckBoxSaturation, VideoProcAmp.Saturation, VideoProcAmp.SaturationRange, VideoProcAmp.SaturationAuto);

      if VideoProcAmp.SharpnessSupported then
        SetTrackBar(LabelSharpness, TrackBarSharpness, CheckBoxSharpness, VideoProcAmp.Sharpness, VideoProcAmp.SharpnessRange, VideoProcAmp.SharpnessAuto);

      if VideoProcAmp.WhiteBalanceSupported then
        SetTrackBar(LabelWhiteBalance, TrackBarWhiteBalance, CheckBoxWhiteBalance, VideoProcAmp.WhiteBalance, VideoProcAmp.WhiteBalanceRange, VideoProcAmp.WhiteBalanceAuto);
    end;

    CameraControl := VideoCapture.CameraControl;
    if CameraControl <> nil then
    begin
      if CameraControl.ExposureSupported then
        SetTrackBar(LabelExposure, TrackBarExposure, CheckBoxExposure, CameraControl.Exposure, CameraControl.ExposureRange, CameraControl.ExposureAuto);

      if CameraControl.FocusSupported then
        SetTrackBar(LabelFocus, TrackBarFocus, CheckBoxFocus, CameraControl.Focus, CameraControl.FocusRange, CameraControl.FocusAuto);

      if CameraControl.IrisSupported then
        SetTrackBar(LabelIris, TrackBarIris, CheckBoxIris, CameraControl.Iris, CameraControl.IrisRange, CameraControl.IrisAuto);

      if CameraControl.PanSupported then
        SetTrackBar(LabelPan, TrackBarPan, CheckBoxPan, CameraControl.Pan, CameraControl.PanRange, CameraControl.PanAuto);

      if CameraControl.RollSupported then
        SetTrackBar(LabelRoll, TrackBarRoll, CheckBoxRoll, CameraControl.Roll, CameraControl.RollRange, CameraControl.RollAuto);

      if CameraControl.TiltSupported then
        SetTrackBar(LabelTilt, TrackBarTilt, CheckBoxTilt, CameraControl.Tilt, CameraControl.TiltRange, CameraControl.TiltAuto);

      if CameraControl.ZoomSupported then
        SetTrackBar(LabelZoom, TrackBarZoom, CheckBoxZoom, CameraControl.Zoom, CameraControl.ZoomRange, CameraControl.ZoomAuto);
    end;
  finally
    TrackBarsSetting := False;
  end;
end;

procedure TFormMain.CheckBoxClick(Sender: TObject);
var
  VideoProcAmp: IVideoProcAmp;
  CameraControl: ICameraControl;
begin
  if not TrackBarsSetting then
  try
    VideoProcAmp := VideoCapture.VideoProcAmp;
    if VideoProcAmp <> nil then
    begin
      if Sender = CheckBoxBacklightCompensation then
        VideoProcAmp.BacklightCompensationAuto := CheckBoxBacklightCompensation.Checked
      else if Sender = CheckBoxBrightness then
        VideoProcAmp.BrightnessAuto := CheckBoxBrightness.Checked
      else if Sender = CheckBoxColorEnable then
        VideoProcAmp.ColorEnableAuto := CheckBoxColorEnable.Checked
      else if Sender = CheckBoxContrast then
        VideoProcAmp.ContrastAuto := CheckBoxContrast.Checked
      else if Sender = CheckBoxGain then
        VideoProcAmp.GainAuto := CheckBoxGain.Checked
      else if Sender = CheckBoxGamma then
        VideoProcAmp.GammaAuto := CheckBoxGamma.Checked
      else if Sender = CheckBoxHue then
        VideoProcAmp.HueAuto := CheckBoxHue.Checked
      else if Sender = CheckBoxSaturation then
        VideoProcAmp.SaturationAuto := CheckBoxSaturation.Checked
      else if Sender = CheckBoxSharpness then
        VideoProcAmp.SharpnessAuto := CheckBoxSharpness.Checked
      else if Sender = CheckBoxWhiteBalance then
        VideoProcAmp.WhiteBalanceAuto := CheckBoxWhiteBalance.Checked
    end;

    CameraControl := VideoCapture.CameraControl;
    if CameraControl <> nil then
    begin
      if Sender = CheckBoxExposure then
        CameraControl.ExposureAuto := CheckBoxExposure.Checked
      else if Sender = CheckBoxFocus then
        CameraControl.FocusAuto := CheckBoxFocus.Checked
      else if Sender = CheckBoxIris then
        CameraControl.IrisAuto := CheckBoxIris.Checked
      else if Sender = CheckBoxPan then
        CameraControl.PanAuto := CheckBoxPan.Checked
      else if Sender = CheckBoxRoll then
        CameraControl.RollAuto := CheckBoxRoll.Checked
      else if Sender = CheckBoxTilt then
        CameraControl.TiltAuto := CheckBoxTilt.Checked
      else if Sender = CheckBoxZoom then
        CameraControl.ZoomAuto := CheckBoxZoom.Checked;
    end;
  finally
    SetTrackBars;
  end
end;

procedure TFormMain.TrackBarChange(Sender: TObject);
var
  VideoProcAmp: IVideoProcAmp;
  CameraControl: ICameraControl;
begin
  if not TrackBarsSetting then
  begin
    VideoProcAmp := VideoCapture.VideoProcAmp;
    if VideoProcAmp <> nil then
    begin
      if Sender = TrackBarBacklightCompensation then
        VideoProcAmp.BacklightCompensation := TrackBarBacklightCompensation.Position
      else if Sender = TrackBarBrightness then
        VideoProcAmp.Brightness := TrackBarBrightness.Position
      else if Sender = TrackBarColorEnable then
        VideoProcAmp.ColorEnable := TrackBarColorEnable.Position
      else if Sender = TrackBarContrast then
        VideoProcAmp.Contrast := TrackBarContrast.Position
      else if Sender = TrackBarGain then
        VideoProcAmp.Gain := TrackBarGain.Position
      else if Sender = TrackBarGamma then
        VideoProcAmp.Gamma := TrackBarGamma.Position
      else if Sender = TrackBarHue then
        VideoProcAmp.Hue := TrackBarHue.Position
      else if Sender = TrackBarSaturation then
        VideoProcAmp.Saturation := TrackBarSaturation.Position
      else if Sender = TrackBarSharpness then
        VideoProcAmp.Sharpness := TrackBarSharpness.Position
      else if Sender = TrackBarWhiteBalance then
        VideoProcAmp.WhiteBalance := TrackBarWhiteBalance.Position
    end;

    CameraControl := VideoCapture.CameraControl;
    if CameraControl <> nil then
    begin
      if Sender = TrackBarExposure then
        CameraControl.Exposure := TrackBarExposure.Position
      else if Sender = TrackBarFocus then
        CameraControl.Focus := TrackBarFocus.Position
      else if Sender = TrackBarIris then
        CameraControl.Iris := TrackBarIris.Position
      else if Sender = TrackBarPan then
        CameraControl.Pan := TrackBarPan.Position
      else if Sender = TrackBarRoll then
        CameraControl.Roll := TrackBarRoll.Position
      else if Sender = TrackBarTilt then
        CameraControl.Tilt := TrackBarTilt.Position
      else if Sender = TrackBarZoom then
        CameraControl.Zoom := TrackBarZoom.Position;
    end;
  end;
end;

procedure TFormMain.ButtonDefaultValuesClick(Sender: TObject);
var
  VideoProcAmp: IVideoProcAmp;
  CameraControl: ICameraControl;
begin
  try
    VideoProcAmp := VideoCapture.VideoProcAmp;
    if VideoProcAmp <> nil then
    begin
      if VideoProcAmp.BacklightCompensationSupported then
        if not VideoProcAmp.BacklightCompensationAuto then
          VideoProcAmp.BacklightCompensation := VideoProcAmp.BacklightCompensationRange.Default;

      if VideoProcAmp.BrightnessSupported then
        if not VideoProcAmp.BrightnessAuto then
          VideoProcAmp.Brightness := VideoProcAmp.BrightnessRange.Default;

      if VideoProcAmp.ColorEnableSupported then
        if not VideoProcAmp.ColorEnableAuto then
          VideoProcAmp.ColorEnable := VideoProcAmp.ColorEnableRange.Default;

      if VideoProcAmp.ContrastSupported then
        if not VideoProcAmp.ContrastAuto then
          VideoProcAmp.Contrast := VideoProcAmp.ContrastRange.Default;

      if VideoProcAmp.GainSupported then
        if not VideoProcAmp.GainAuto then
          VideoProcAmp.Gain := VideoProcAmp.GainRange.Default;

      if VideoProcAmp.GammaSupported then
        if not VideoProcAmp.GammaAuto then
          VideoProcAmp.Gamma := VideoProcAmp.GammaRange.Default;

      if VideoProcAmp.HueSupported then
        if not VideoProcAmp.HueAuto then
          VideoProcAmp.Hue := VideoProcAmp.HueRange.Default;

      if VideoProcAmp.SaturationSupported then
        if not VideoProcAmp.SaturationAuto then
          VideoProcAmp.Saturation := VideoProcAmp.SaturationRange.Default;

      if VideoProcAmp.SharpnessSupported then
        if not VideoProcAmp.SharpnessAuto then
          VideoProcAmp.Sharpness := VideoProcAmp.SharpnessRange.Default;

      if VideoProcAmp.WhiteBalanceSupported then
        if not VideoProcAmp.WhiteBalanceAuto then
          VideoProcAmp.WhiteBalance := VideoProcAmp.WhiteBalanceRange.Default;
    end;

    CameraControl := VideoCapture.CameraControl;
    if CameraControl <> nil then
    begin
      if CameraControl.ExposureSupported then
        if not CameraControl.ExposureAuto then
          CameraControl.Exposure := CameraControl.ExposureRange.Default;

      if CameraControl.FocusSupported then
        if not CameraControl.FocusAuto then
          CameraControl.Focus := CameraControl.FocusRange.Default;

      if CameraControl.IrisSupported then
        if not CameraControl.IrisAuto then
          CameraControl.Iris := CameraControl.IrisRange.Default;

      if CameraControl.PanSupported then
        if not CameraControl.PanAuto then
          CameraControl.Pan := CameraControl.PanRange.Default;

      if CameraControl.RollSupported then
        if not CameraControl.RollAuto then
          CameraControl.Roll := CameraControl.RollRange.Default;

      if CameraControl.TiltSupported then
        if not CameraControl.TiltAuto then
          CameraControl.Tilt := CameraControl.TiltRange.Default;

      if CameraControl.ZoomSupported then
        if CameraControl.ZoomAuto then
          CameraControl.Zoom := CameraControl.ZoomRange.Default;
    end;
  finally
    SetTrackBars;
  end
end;

end.
