unit Main;

{$MODE Delphi}

interface

uses
  LCLIntf, LCLType, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ExtDlgs, BarcodeSuite, ComCtrls;

type
  TFormMain = class(TForm)
    Image: TImage;
    Memo: TMemo;
    OpenPictureDialog: TOpenPictureDialog;
    Splitter: TSplitter;
    PanelLeft: TPanel;
    ButtonSelectPicture: TButton;
    PanelRight: TPanel;
    CheckBoxAztec: TCheckBox;
    CheckBoxCodabar: TCheckBox;
    CheckBoxCode39: TCheckBox;
    CheckBoxCode93: TCheckBox;
    PaintBox: TPaintBox;
    CheckBoxCode128: TCheckBox;
    CheckBoxDataBar: TCheckBox;
    CheckBoxDataBarExpanded: TCheckBox;
    CheckBoxDataMatrix: TCheckBox;
    CheckBoxEan8: TCheckBox;
    CheckBoxEan13: TCheckBox;
    CheckBoxItf: TCheckBox;
    CheckBoxMaxiCode: TCheckBox;
    CheckBoxPdf417: TCheckBox;
    CheckBoxQrCode: TCheckBox;
    CheckBoxUpcA: TCheckBox;
    CheckBoxUpcE: TCheckBox;
    CheckBoxMicroQrCode: TCheckBox;
    ComboBoxBinarizer: TComboBox;
    LabelBinarizer: TLabel;
    ComboBoxEanAddOn: TComboBox;
    LabelEanAddOn: TLabel;
    CheckBoxTryHarder: TCheckBox;
    CheckBoxTryRotate: TCheckBox;
    CheckBoxTryDownscale: TCheckBox;
    CheckBoxPureBarcode: TCheckBox;
    CheckBoxShowPosition: TCheckBox;
    CheckBoxCodaBarStartEnd: TCheckBox;
    CheckBoxValidateItf: TCheckBox;
    CheckBoxValidateCode39: TCheckBox;
    CheckBoxCode39Extended: TCheckBox;
    EditMinLineCount: TEdit;
    LabelMinLineCount: TLabel;
    UpDownMinLineCount: TUpDown;
    LabelMaxBarcodes: TLabel;
    EditMaxBarcodes: TEdit;
    UpDownMaxBarcodes: TUpDown;
    CheckBoxTryInvert: TCheckBox;
    LabelTextMode: TLabel;
    ComboBoxTextMode: TComboBox;
    procedure ButtonSelectPictureClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CheckBoxAztecClick(Sender: TObject);
    procedure CheckBoxCodabarClick(Sender: TObject);
    procedure CheckBoxCode39Click(Sender: TObject);
    procedure CheckBoxCode93Click(Sender: TObject);
    procedure PaintBoxPaint(Sender: TObject);
    procedure CheckBoxDataBarClick(Sender: TObject);
    procedure CheckBoxCode128Click(Sender: TObject);
    procedure CheckBoxDataBarExpandedClick(Sender: TObject);
    procedure CheckBoxDataMatrixClick(Sender: TObject);
    procedure CheckBoxEan8Click(Sender: TObject);
    procedure CheckBoxEan13Click(Sender: TObject);
    procedure CheckBoxItfClick(Sender: TObject);
    procedure CheckBoxMaxiCodeClick(Sender: TObject);
    procedure CheckBoxPdf417Click(Sender: TObject);
    procedure CheckBoxQrCodeClick(Sender: TObject);
    procedure CheckBoxUpcAClick(Sender: TObject);
    procedure CheckBoxUpcEClick(Sender: TObject);
    procedure CheckBoxMicroQrCodeClick(Sender: TObject);
    procedure ComboBoxBinarizerChange(Sender: TObject);
    procedure ComboBoxEanAddOnChange(Sender: TObject);
    procedure CheckBoxTryHarderClick(Sender: TObject);
    procedure CheckBoxTryRotateClick(Sender: TObject);
    procedure CheckBoxTryDownscaleClick(Sender: TObject);
    procedure CheckBoxPureBarcodeClick(Sender: TObject);
    procedure CheckBoxShowPositionClick(Sender: TObject);
    procedure CheckBoxCodaBarStartEndClick(Sender: TObject);
    procedure CheckBoxValidateItfClick(Sender: TObject);
    procedure CheckBoxValidateCode39Click(Sender: TObject);
    procedure CheckBoxCode39ExtendedClick(Sender: TObject);
    procedure EditMinLineCountChange(Sender: TObject);
    procedure EditMaxBarcodesChange(Sender: TObject);
    procedure CheckBoxTryInvertClick(Sender: TObject);
    procedure ComboBoxTextModeChange(Sender: TObject);
  private
    { Private declarations }
    BarcodeDecoder: TBarcodeDecoder;
    TopLeft: TPoint;
    TopRight: TPoint;
    BottomLeft: TPoint;
    BottomRight: TPoint;
    procedure DecodeBarcode;
    procedure AddLine(const Text: string = '');
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

uses Types
{$ifdef CONDITIONALEXPRESSIONS}
  {$if CompilerVersion >= 30} // Delphi 10
    , GIFImg, PngImage
  {$ifend}
{$endif}
;

{$R *.lfm}

function BytesToStr(const Value: TByteDynArray): string;
var I: Integer;
begin
  Result := '';
  for I := 0 to Length(Value) - 1 do
    Result := Result + IntToHex(Value[I], 2) + ' ';
end;

function ToStr(ErrorType: TErrorType): string; overload;
begin
  case ErrorType of
    etFormat:      Result := 'Format error';
    etChecksum:    Result := 'Checksum error';
    etUnsupported: Result := 'Unsupported';
    else {etNone}  Result := 'No error';
  end;
end;

function ToStr(ContentType: TContentType): string; overload;
begin
  case ContentType of
    ctBinary:     Result := 'Binary';
    ctMixed:      Result := 'Mixed';
    ctGS1:        Result := 'GS1';
    ctISO15434:   Result := 'ISO 15434';
    ctUnknownECI: Result := 'Unknown ECI';
    else {ctText} Result := 'Text';
  end;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  BarcodeDecoder := TBarcodeDecoder.Create;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  BarcodeDecoder.Free;
end;

procedure TFormMain.AddLine(const Text: string = '');
begin
  Memo.Lines.Add(Text);
end;

procedure TFormMain.ButtonSelectPictureClick(Sender: TObject);
begin
  with OpenPictureDialog do
    if Execute then
    begin
      Image.Picture.LoadFromFile(FileName);
      DecodeBarcode;
    end;
end;

procedure TFormMain.DecodeBarcode;
var
  Barcodes: TDecodeResults;
  I: Integer;
begin
  Memo.Lines.Clear;
  TopLeft.X := 0;
  TopLeft.Y := 0;
  TopRight.X := 0;
  TopRight.Y := 0;
  BottomLeft.X := 0;
  BottomLeft.Y := 0;
  BottomRight.X := 0;
  BottomRight.Y := 0;

  Barcodes := BarcodeDecoder.Decode(Image.Picture.Graphic);
  if Length(Barcodes) = 0 then
    AddLine('No barcode found')
  else
    for I := 0 to Length(Barcodes) - 1 do
      with Barcodes[I] do
      try
        Memo.Lines.BeginUpdate;
        AddLine('Barcode ' + IntToStr(I + 1));
        if ErrorType <> etNone then
          AddLine('  Error: ' + ToStr(ErrorType));
        if ErrorMessage <> '' then
          AddLine('  Error message: ' + ErrorMessage);
        AddLine('  Format: ' + FormatName);
        AddLine('  Text: ' + Text);
        AddLine('  Rendered text: ' + RenderedText);
        AddLine('  Data: ' + BytesToStr(Data));
        AddLine('  Content type: ' + ToStr(ContentType));
        if Version <> '' then
          AddLine('  Version: ' + Version);
        if EccLevel <> '' then
          AddLine('  ECC level: ' + EccLevel);
        AddLine('  Has ECI: ' + BoolToStr(HasEci, True));
        AddLine('  ECI data: ' + BytesToStr(EciData));
        AddLine('  Position: '
         + '(' + IntToStr(TopLeft.X) + ',' + IntToStr(TopLeft.Y)
         + ') (' + IntToStr(TopRight.X) + ',' + IntToStr(TopRight.Y)
         + ') (' + IntToStr(BottomRight.X) + ',' + IntToStr(BottomRight.Y)
         + ') (' + IntToStr(BottomLeft.X) + ',' + IntToStr(BottomLeft.Y) + ')');
        AddLine('  Lines: ' + IntToStr(LineCount));
        AddLine('  Orientation: ' + IntToStr(Orientation));
        AddLine('  Inverted: ' + BoolToStr(Inverted, True));
        AddLine('  Mirrored: ' + BoolToStr(Mirrored, True));
        AddLine('  Symbology identifier: ' + SymbologyIdentifier);
        if SequenceId <> '' then
          AddLine('  Sequence ID: ' + SequenceId);
        if SequenceSize <> -1 then
          AddLine('  Sequence size: ' + IntToStr(SequenceSize));
        if SequenceIndex <> -1 then
          AddLine('  Sequence index: ' + IntToStr(SequenceIndex));
        AddLine;
        if I = 0 then
        begin
          Self.TopLeft := TopLeft;
          Self.TopRight := TopRight;
          Self.BottomLeft := BottomLeft;
          Self.BottomRight := BottomRight;
        end;
      finally
        Memo.Lines.EndUpdate;
      end;

  PaintBox.Invalidate;
end;

procedure TFormMain.CheckBoxAztecClick(Sender: TObject);
begin
  if CheckBoxAztec.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foAztec]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foAztec];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxCodabarClick(Sender: TObject);
begin
  if CheckBoxCodabar.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foCodabar]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foCodabar];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxCode39Click(Sender: TObject);
begin
  if CheckBoxCode39.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foCode39]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foCode39];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxCode93Click(Sender: TObject);
begin
  if CheckBoxCode93.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foCode93]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foCode93];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxCode128Click(Sender: TObject);
begin
  if CheckBoxCode128.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foCode128]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foCode128];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxDataBarClick(Sender: TObject);
begin
  if CheckBoxDataBar.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foDataBar]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foDataBar];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxDataBarExpandedClick(Sender: TObject);
begin
  if CheckBoxDataBarExpanded.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foDataBarExpanded]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foDataBarExpanded];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxDataMatrixClick(Sender: TObject);
begin
  if CheckBoxDataMatrix.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foDataMatrix]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foDataMatrix];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxEan8Click(Sender: TObject);
begin
  if CheckBoxEan8.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foEan8]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foEan8];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxEan13Click(Sender: TObject);
begin
  if CheckBoxEan13.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foEan13]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foEan13];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxItfClick(Sender: TObject);
begin
  if CheckBoxItf.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foItf]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foItf];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxMaxiCodeClick(Sender: TObject);
begin
  if CheckBoxMaxiCode.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foMaxiCode]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foMaxiCode];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxPdf417Click(Sender: TObject);
begin
  if CheckBoxPdf417.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foPdf417]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foPdf417];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxQrCodeClick(Sender: TObject);
begin
  if CheckBoxQrCode.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foQrCode]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foQrCode];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxUpcAClick(Sender: TObject);
begin
  if CheckBoxUpcA.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foUpcA]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foUpcA];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxUpcEClick(Sender: TObject);
begin
  if CheckBoxUpcE.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foUpcE]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foUpcE];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxMicroQrCodeClick(Sender: TObject);
begin
  if CheckBoxMicroQrCode.Checked then
    BarcodeDecoder.Formats := BarcodeDecoder.Formats + [foMicroQrCode]
  else
    BarcodeDecoder.Formats := BarcodeDecoder.Formats - [foMicroQrCode];
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxShowPositionClick(Sender: TObject);
begin
  PaintBox.Invalidate;
end;

procedure TFormMain.ComboBoxTextModeChange(Sender: TObject);
begin
  BarcodeDecoder.TextMode := TTextMode(ComboBoxTextMode.ItemIndex);
  DecodeBarcode;
end;

procedure TFormMain.ComboBoxBinarizerChange(Sender: TObject);
begin
  BarcodeDecoder.Binarizer := TBinarizer(ComboBoxBinarizer.ItemIndex);
  DecodeBarcode;
end;

procedure TFormMain.ComboBoxEanAddOnChange(Sender: TObject);
begin
  BarcodeDecoder.EanAddOn := TEanAddOn(ComboBoxEanAddOn.ItemIndex);
  DecodeBarcode;
end;

procedure TFormMain.EditMinLineCountChange(Sender: TObject);
begin
  BarcodeDecoder.MinLineCount := UpDownMinLineCount.Position;
  DecodeBarcode;
end;

procedure TFormMain.EditMaxBarcodesChange(Sender: TObject);
begin
  BarcodeDecoder.MaxBarcodeCount := UpDownMaxBarcodes.Position;
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxTryHarderClick(Sender: TObject);
begin
  BarcodeDecoder.TryHarder := CheckBoxTryHarder.Checked;
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxTryRotateClick(Sender: TObject);
begin
  BarcodeDecoder.TryRotate := CheckBoxTryRotate.Checked;
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxTryInvertClick(Sender: TObject);
begin
  BarcodeDecoder.TryInvert := CheckBoxTryInvert.Checked;
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxTryDownscaleClick(Sender: TObject);
begin
  BarcodeDecoder.TryDownscale := CheckBoxTryDownscale.Checked;
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxPureBarcodeClick(Sender: TObject);
begin
  BarcodeDecoder.PureBarcode := CheckBoxPureBarcode.Checked;
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxCodaBarStartEndClick(Sender: TObject);
begin
  BarcodeDecoder.ReturnCodabarStartEnd := CheckBoxCodaBarStartEnd.Checked;
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxValidateItfClick(Sender: TObject);
begin
  BarcodeDecoder.ValidateITF := CheckBoxValidateItf.Checked;
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxValidateCode39Click(Sender: TObject);
begin
  BarcodeDecoder.ValidateCode39 := CheckBoxValidateCode39.Checked;
  DecodeBarcode;
end;

procedure TFormMain.CheckBoxCode39ExtendedClick(Sender: TObject);
begin
  BarcodeDecoder.TryCode39ExtendedMode := CheckBoxCode39Extended.Checked;
  DecodeBarcode;
end;

procedure TFormMain.PaintBoxPaint(Sender: TObject);
var X, Y: Integer;
begin
  if CheckBoxShowPosition.Checked then
  begin
    X := (PaintBox.Width - Image.Picture.Width) div 2;
    Y := (PaintBox.Height - Image.Picture.Height) div 2;
    with PaintBox.Canvas do
    begin
      Pen.Color := clLime;
      Pen.Width := 2;
      MoveTo(X + TopLeft.X, Y + TopLeft.Y);
      LineTo(X + TopRight.X, Y + TopRight.Y);
      LineTo(X + BottomRight.X, Y + BottomRight.Y);
      LineTo(X + BottomLeft.X, Y + BottomLeft.Y);
      LineTo(X + TopLeft.X, Y + TopLeft.Y);
    end;
  end;
end;

end.
