unit Main;

{$MODE Delphi}

interface

uses
  LCLIntf, LCLType, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, BarcodeSuite, ExtCtrls, StdCtrls, ExtDlgs, ComCtrls, ColorBox;

type
  TFormMain = class(TForm)
    PanelTop: TPanel;
    PanelBottom: TPanel;
    Image: TImage;
    ComboBoxFormat: TComboBox;
    LabelFormat: TLabel;
    MemoText: TMemo;
    ColorBoxBackColor: TColorBox;
    ColorBoxForeColor: TColorBox;
    LabelBackColor: TLabel;
    LabelForeColor: TLabel;
    ComboBoxPixelFormat: TComboBox;
    LabelPixelFormat: TLabel;
    ButtonSave: TButton;
    SavePictureDialog: TSavePictureDialog;
    ComboBoxEccLevel: TComboBox;
    LabelEccLevel: TLabel;
    EditMargin: TEdit;
    UpDownMargin: TUpDown;
    LabelMargin: TLabel;
    Label1: TLabel;
    EditHeight: TEdit;
    UpDownHeight: TUpDown;
    Label2: TLabel;
    EditWidth: TEdit;
    UpDownWidth: TUpDown;
    Label3: TLabel;
    ComboBoxCharacterSet: TComboBox;
    StatusBar: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ComboBoxFormatChange(Sender: TObject);
    procedure MemoTextChange(Sender: TObject);
    procedure ColorBoxBackColorChange(Sender: TObject);
    procedure ColorBoxForeColorChange(Sender: TObject);
    procedure ComboBoxPixelFormatChange(Sender: TObject);
    procedure ButtonSaveClick(Sender: TObject);
    procedure ComboBoxEccLevelChange(Sender: TObject);
    procedure EditMarginChange(Sender: TObject);
    procedure EditHeightChange(Sender: TObject);
    procedure EditWidthChange(Sender: TObject);
    procedure ComboBoxCharacterSetChange(Sender: TObject);
  private
    { Private declarations }
    BarcodeEncoder: TBarcodeEncoder;
    Bitmap: TBitmap;
    procedure DrawBarcode;
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.lfm}

procedure TFormMain.FormCreate(Sender: TObject);
begin
  BarcodeEncoder := TBarcodeEncoder.Create;
  BarcodeEncoder.Text := MemoText.Text;
  BarcodeEncoder.Height := UpDownHeight.Position;
  BarcodeEncoder.Width := UpDownWidth.Position;

  Bitmap := TBitmap.Create;
  Bitmap.PixelFormat := pf24bit;

  DrawBarcode;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  Bitmap.Free;
  BarcodeEncoder.Free;
end;

procedure TFormMain.ComboBoxFormatChange(Sender: TObject);
begin
  BarcodeEncoder.Format := TFormat(ComboBoxFormat.ItemIndex + 1);
  DrawBarcode;
end;

procedure TFormMain.ColorBoxBackColorChange(Sender: TObject);
begin
  BarcodeEncoder.BackColor := ColorBoxBackColor.Selected;
  DrawBarcode;
end;

procedure TFormMain.ColorBoxForeColorChange(Sender: TObject);
begin
  BarcodeEncoder.ForeColor := ColorBoxForeColor.Selected;
  DrawBarcode;
end;

procedure TFormMain.ComboBoxPixelFormatChange(Sender: TObject);
begin
  Bitmap.PixelFormat := pfDevice; // reset bitmap
  case ComboBoxPixelFormat.ItemIndex of
    0: Bitmap.PixelFormat := pf1bit;
    1: Bitmap.PixelFormat := pf24bit;
    2: Bitmap.PixelFormat := pf32bit;
  end;
  DrawBarcode;
end;

procedure TFormMain.ComboBoxCharacterSetChange(Sender: TObject);
begin
  BarcodeEncoder.CharacterSet := TCharacterSet(ComboBoxCharacterSet.ItemIndex);
  DrawBarcode;
end;

procedure TFormMain.ComboBoxEccLevelChange(Sender: TObject);
begin
  BarcodeEncoder.EccLevel := TEccLevel(ComboBoxEccLevel.ItemIndex);
  DrawBarcode;
end;

procedure TFormMain.EditMarginChange(Sender: TObject);
begin
  BarcodeEncoder.Margin := UpDownMargin.Position;
  DrawBarcode;
end;

procedure TFormMain.EditHeightChange(Sender: TObject);
begin
  BarcodeEncoder.Height := UpDownHeight.Position;
  DrawBarcode;
end;

procedure TFormMain.EditWidthChange(Sender: TObject);
begin
  BarcodeEncoder.Width := UpDownWidth.Position;
  DrawBarcode;
end;

procedure TFormMain.ButtonSaveClick(Sender: TObject);
begin
  if SavePictureDialog.Execute then
    Bitmap.SaveToFile(SavePictureDialog.FileName);
end;

procedure TFormMain.MemoTextChange(Sender: TObject);
begin
  BarcodeEncoder.Text := MemoText.Text;
  DrawBarcode;
end;

procedure TFormMain.DrawBarcode;
var EncodeResult: TEncodeResult;
begin
  try
    EncodeResult := BarcodeEncoder.Encode(Bitmap);
    Image.Picture.Assign(Bitmap);
    StatusBar.SimpleText := EncodeResult.ErrorMessage;
  except
    on E: Exception do
      StatusBar.SimpleText := E.Message;
  end;
end;

end.
